.s3file {
  display: inline-block;
  width: 220px;

  .progress {
    background: #333;
  }
  .progress,
  .link,
  .remove,
  input {
    display: none;
  }
}

.s3file.progress-active .progress {
  display: block;
}

.s3file.link-active {
  .link {
    display: block;
  }

  .remove {
    display: inline-block;
  }
}

.s3file.form-active input {
  display: block;
}
